@@ -9,128 +9,275 @@ import torch
 import torch.nn as nn
 from tensorflow.keras.layers import Dense
 from tensorflow.keras.models import Sequential
-from thingsvision.utils.data import DataLoader, ImageDataset
+
 from thingsvision import get_extractor
+from thingsvision.utils.data import DataLoader, ImageDataset
+from torch.utils.data import Subset, Dataset
 
 DATA_PATH = "./data"
 TEST_PATH = "./test_images"
 OUT_PATH = "./test"
 
-SSL_RN50_DEFAULT_CONFIG = {
-    "modules": ["avgpool"],
-    "pretrained": True,
-    "source": "ssl",
-}
-
 MODEL_AND_MODULE_NAMES = {
     # Torchvision models
     "vgg16": {
+        "model_name": "vgg16",
         "modules": ["features.23", "classifier.3"],
         "pretrained": True,
         "source": "torchvision",
     },
     "vgg19_bn": {
+        "model_name": "vgg19_bn",
         "modules": ["features.23", "classifier.3"],
         "pretrained": False,
         "source": "torchvision",
     },
+    "vit_b_16": {
+        "model_name": "vit_b_16",
+        "modules": ["encoder.ln"],
+        "pretrained": True,
+        "source": "torchvision",
+        "kwargs": {"token_extraction": "cls_token", "weights": "DEFAULT"},
+    },
     # Hardcoded models
     "cornet_r": {
+        "model_name": "cornet_r",
         "modules": ["decoder.flatten"],
         "pretrained": True,
         "source": "custom",
     },
     "cornet_rt": {
+        "model_name": "cornet_rt",
         "modules": ["decoder.flatten"],
-        "pretrained": False,
+        "pretrained": True,
         "source": "custom",
     },
     "cornet_s": {
+        "model_name": "cornet_s",
         "modules": ["decoder.flatten"],
         "pretrained": False,
         "source": "custom",
     },
     "cornet_z": {
+        "model_name": "cornet_z",
         "modules": ["decoder.flatten"],
         "pretrained": True,
         "source": "custom",
     },
     # Custom models
     "VGG16_ecoset": {
+        "model_name": "VGG16_ecoset",
         "modules": ["classifier.3"],
         "pretrained": True,
         "source": "custom",
     },
-    "clip": {
-        "modules": ["visual"],
-        "pretrained": True,
-        "source": "custom",
-        "clip": True,
-        "kwargs": {"variant": "ViT-B/32"},
-    },
-    "clip": {
+    "clip_rn50": {
+        "model_name": "clip",
         "modules": ["visual"],
         "pretrained": True,
         "source": "custom",
         "clip": True,
         "kwargs": {"variant": "RN50"},
     },
-    "OpenCLIP": {
+    "OpenCLIP_vitl14": {
+        "model_name": "OpenCLIP",
         "modules": ["visual"],
         "pretrained": True,
         "source": "custom",
         "clip": True,
-        "kwargs": {"variant": "ViT-B-32", "dataset": "openai"},
+        "kwargs": {"variant": "ViT-L-14", "dataset": "laion400m_e32"},
+    },
+    "OpenCLIP_vitl14_custom_vision_cfg_avg_pool": {
+        "model_name": "OpenCLIP",
+        "modules": ["visual"],
+        "pretrained": True,
+        "source": "custom",
+        "clip": True,
+        "kwargs": {
+            "variant": "ViT-L-14",
+            "dataset": "laion400m_e32",
+            "vision_cfg": {
+                "image_size": 224,
+                "layers": 24,
+                "width": 1024,
+                "patch_size": 14,
+                "pool_type": "none",
+            },
+            "token_extraction": "avg_pool",
+        },
+    },
+    "OpenCLIP_vitl14_custom_vision_cfg_cls_token": {
+        "model_name": "OpenCLIP",
+        "modules": ["visual"],
+        "pretrained": True,
+        "source": "custom",
+        "clip": True,
+        "kwargs": {
+            "variant": "ViT-L-14",
+            "dataset": "laion400m_e32",
+            "vision_cfg": {
+                "image_size": 224,
+                "layers": 24,
+                "width": 1024,
+                "patch_size": 14,
+                "pool_type": "none",
+            },
+            "token_extraction": "cls_token",
+        },
     },
     # Timm models
-    "mixnet_l": {"modules": ["conv_head"], "pretrained": True, "source": "timm"},
-    "gluon_inception_v3": {
-        "modules": ["Mixed_6d"],
-        "pretrained": False,
+    "vit_base_patch16_224": {
+        "model_name": "vit_base_patch16_224",
+        "modules": ["norm"],
+        "pretrained": True,
+        "source": "timm",
+        "kwargs": {"token_extraction": "avg_pool"},
+    },
+    "mixnet_l": {
+        "model_name": "mixnet_l",
+        "modules": ["conv_head"],
+        "pretrained": True,
         "source": "timm",
     },
     # Keras models
-    "VGG16": {
+    "VGG16_keras": {
+        "model_name": "VGG16",
         "modules": ["block1_conv1", "flatten"],
         "pretrained": True,
         "source": "keras",
     },
-    "VGG19": {
-        "modules": ["block1_conv1", "flatten"],
-        "pretrained": False,
-        "source": "keras",
-    },
     # Vissl models
-    'simclr-rn50': SSL_RN50_DEFAULT_CONFIG,
-    'mocov2-rn50': SSL_RN50_DEFAULT_CONFIG,
-    'jigsaw-rn50': SSL_RN50_DEFAULT_CONFIG,
-    'rotnet-rn50': SSL_RN50_DEFAULT_CONFIG,
-    'swav-rn50': SSL_RN50_DEFAULT_CONFIG,
-    'pirl-rn50': SSL_RN50_DEFAULT_CONFIG,
-    'barlowtwins-rn50': SSL_RN50_DEFAULT_CONFIG,
-    'vicreg-rn50': SSL_RN50_DEFAULT_CONFIG,
-    'dino-rn50' : SSL_RN50_DEFAULT_CONFIG,
-    # Harmonization models
-    "Harmonization": {
-        "modules": ["visual"],
+    "simclr-rn50": {
+        "model_name": "simclr-rn50",
+        "modules": ["avgpool"],
+        "pretrained": True,
+        "source": "ssl",
+    },
+    "mocov2-rn50": {
+        "model_name": "mocov2-rn50",
+        "modules": ["avgpool"],
+        "pretrained": True,
+        "source": "ssl",
+    },
+    "jigsaw-rn50": {
+        "model_name": "jigsaw-rn50",
+        "modules": ["avgpool"],
+        "pretrained": True,
+        "source": "ssl",
+    },
+    "swav-rn50": {
+        "model_name": "swav-rn50",
+        "modules": ["avgpool"],
+        "pretrained": True,
+        "source": "ssl",
+    },
+    "barlowtwins-rn50": {
+        "model_name": "barlowtwins-rn50",
+        "modules": ["avgpool"],
+        "pretrained": True,
+        "source": "ssl",
+    },
+    "dino-vit-tiny-p8": {
+        "model_name": "dino-vit-small-p8",
+        "modules": ["norm"],
+        "pretrained": True,
+        "source": "ssl",
+        "kwargs": {"token_extraction": "cls_token+avg_pool"},
+    },
+    "dino-vit-small-p8": {
+        "model_name": "dino-vit-small-p8",
+        "modules": ["norm"],
+        "pretrained": True,
+        "source": "ssl",
+        "kwargs": {"token_extraction": "avg_pool"},
+    },
+    "dino-vit-base-p8": {
+        "model_name": "dino-vit-base-p8",
+        "modules": ["norm"],
+        "pretrained": True,
+        "source": "ssl",
+        "kwargs": {"token_extraction": "cls_token"},
+    },
+    "dinov2-vit-small-p14": {
+        "model_name": "dinov2-vit-small-p14",
+        "modules": ["norm"],
+        "pretrained": True,
+        "source": "ssl",
+        "kwargs": {"token_extraction": "cls_token+avg_pool"},
+    },
+    "dinov2-vit-base-p14": {
+        "model_name": "dinov2-vit-base-p14",
+        "modules": ["norm"],
+        "pretrained": True,
+        "source": "ssl",
+        "kwargs": {"token_extraction": "cls_token"},
+    },
+    "mae-vit-base-p16": {
+        "model_name": "mae-vit-base-p16",
+        "modules": ["norm", "fc_norm"],
+        "pretrained": True,
+        "source": "ssl",
+        "kwargs": {"token_extraction": "avg_pool"},
+    },
+    # Additional models
+    "Harmonization_visual_ResNet50": {
+        "model_name": "Harmonization",
+        "modules": ["avg_pool"],
         "pretrained": True,
         "source": "custom",
         "kwargs": {"variant": "ResNet50"},
     },
-    "Harmonization": {
-        "modules": ["fc2"],
-        "pretrained": True,
-        "source": "custom",
-        "kwargs": {"variant": "VGG16"},
-    },
-     "Harmonization": {
+    "Harmonization_head_ViT_B16": {
+        "model_name": "Harmonization",
         "modules": ["head"],
         "pretrained": True,
         "source": "custom",
         "kwargs": {"variant": "ViT_B16"},
-    }
+    },
+    "DreamSim_mlp_clip_vitb32": {
+        "model_name": "DreamSim",
+        "modules": ["model.mlp"],
+        "pretrained": True,
+        "source": "custom",
+        "kwargs": {"variant": "clip_vitb32"},
+    },
+    "DreamSim_mlp_dino_vitb16": {
+        "model_name": "DreamSim",
+        "modules": ["model.mlp"],
+        "pretrained": True,
+        "source": "custom",
+        "kwargs": {"variant": "dino_vitb16"},
+    },
+    "SegmentAnything_vit_b": {
+        "model_name": "SegmentAnything",
+        "modules": ["flatten"],
+        "pretrained": True,
+        "source": "custom",
+        "kwargs": {"variant": "vit_b"},
+        # model input size is large, therefore reduce test case on cpu
+        "batch_size": 1,
+        "num_samples": 1,
+    },
+    "kakobrain_align_model": {
+        "model_name": "Kakaobrain_Align",
+        "modules": ["pooler"],
+        "pretrained": True,
+        "source": "custom",
+    },
 }
 
+ALIGNED_MODELS = {
+    "alexnet": "classifier.4",
+    "vgg16": "classifier.3",
+    "resnet18": "avgpool",
+    "resnet50": "avgpool",
+    "clip": "visual",
+    "OpenCLIP": "visual",
+    "dinov2-vit-base-p14": "norm",
+    "dinov2-vit-large-p14": "norm",
+    "dino-vit-base-p16": "norm",
+    "dino-vit-base-p8": "norm",
+}
 
 FILE_FORMATS = ["hdf5", "npy", "mat", "pt", "txt"]
 DISTANCES = ["correlation", "cosine", "euclidean", "gaussian"]
@@ -141,7 +288,6 @@ NUM_OBJECTS = 1854
 NUM_SAMPLES = int(BATCH_SIZE * 2)
 DEVICE = "cuda" if torch.cuda.is_available() else "cpu"
 
-
 tf_model = Sequential()
 tf_model.add(Dense(2, input_dim=1, activation="relu", use_bias=False, name="relu"))
 weights = np.array([[[1, 1]]])
@@ -202,7 +348,6 @@ class SimpleDataset(object):
         self.backend = backend
 
     def __getitem__(self, idx: int) -> Tuple[Any, Any]:
-        target = 0
         value = self.values[idx]
 
         if self.backend == "pt":
@@ -216,22 +361,61 @@ class SimpleDataset(object):
         return len(self.values)
 
 
+class MockImageDataset(Dataset):
+    """Mock dataset that returns (image, label) tuples"""
+
+    def __init__(self, size=10):
+        self.size = size
+
+    def __len__(self):
+        return self.size
+
+    def __getitem__(self, idx):
+        image = torch.randn(3, 32, 32)
+        label = torch.tensor(idx % 5)
+        return image, label
+
+
+class MockImageOnlyDataset(Dataset):
+    """Mock dataset that returns only images (not tuples)"""
+
+    def __init__(self, size=10):
+        self.size = size
+
+    def __len__(self):
+        return self.size
+
+    def __getitem__(self, idx):
+        # Return only image tensor (no tuple)
+        return torch.randn(3, 32, 32)
+
+
 def iterate_through_all_model_combinations():
-    for model_name in MODEL_AND_MODULE_NAMES:
-        pretrained = MODEL_AND_MODULE_NAMES[model_name]["pretrained"]
-        source = MODEL_AND_MODULE_NAMES[model_name]["source"]
-        kwargs = MODEL_AND_MODULE_NAMES[model_name].get("kwargs", {})
+    for model_config in MODEL_AND_MODULE_NAMES.values():
+        model_name = model_config["model_name"]
+        pretrained = model_config["pretrained"]
+        source = model_config["source"]
+        kwargs = model_config.get("kwargs", {})
+
+        # we can set batch size and num_samples to reduce load for large models requiring a large image size
+        batch_size = model_config.get("batch_size", BATCH_SIZE)
+        num_samples = model_config.get("num_samples", NUM_SAMPLES)
+
         extractor, dataset, batches = create_extractor_and_dataloader(
-            model_name, pretrained, source, kwargs
+            model_name, pretrained, source, kwargs, batch_size, num_samples
         )
 
-        modules = MODEL_AND_MODULE_NAMES[model_name]["modules"]
-        clip = MODEL_AND_MODULE_NAMES[model_name].get("clip", False)
-        yield extractor, dataset, batches, modules, model_name, clip
+        modules = model_config["modules"]
+        yield extractor, dataset, batches, modules, model_name
 
 
 def create_extractor_and_dataloader(
-    model_name: str, pretrained: bool, source: str, kwargs: dict = {}
+    model_name: str,
+    pretrained: bool,
+    source: str,
+    kwargs: dict = {},
+    batch_size: int = BATCH_SIZE,
+    num_samples: int = NUM_SAMPLES,
 ):
     """Iterate through models and create model, dataset and data loader."""
     extractor = get_extractor(
@@ -247,9 +431,16 @@ def create_extractor_and_dataloader(
         backend=extractor.get_backend(),
         transforms=extractor.get_transformations(),
     )
+    if num_samples > NUM_SAMPLES:
+        raise ValueError(
+            "\nNumber of samples in test case cannot be larger than the default value, only smaller.\n"
+        )
+    elif num_samples < NUM_SAMPLES:
+        dataset = Subset(dataset, np.arange(num_samples))
+
     batches = DataLoader(
         dataset,
-        batch_size=BATCH_SIZE,
+        batch_size=batch_size,
         backend=extractor.get_backend(),
     )
     return extractor, dataset, batches
@@ -282,6 +473,6 @@ def create_test_images(n_samples: int = NUM_SAMPLES) -> None:
             noisy_img = test_img + np.random.randn(H, W, C)
             noisy_img = noisy_img.astype(np.uint8)
             imageio.imsave(
-                os.path.join(TEST_PATH, cls, f"test_img_{i+1:03d}.png"), noisy_img
+                os.path.join(TEST_PATH, cls, f"test_img_{i + 1:03d}.png"), noisy_img
             )
         print("\n...Successfully created image dataset for testing.\n")
