@@ -1,13 +1,13 @@
 import os
 import shutil
 import unittest
-import torch
 
 import numpy as np
-from thingsvision.core.extraction import center_features, normalize_features
-from thingsvision.utils.storing import save_features, split_features
+import torch
 
 import tests.helper as helper
+from thingsvision.core.extraction import center_features, normalize_features
+from thingsvision.utils.storing import save_features, split_features
 
 
 class FeaturesTestCase(unittest.TestCase):
@@ -45,6 +45,19 @@ class FeaturesTestCase(unittest.TestCase):
         )
         return features
 
+    def get_multi_features(self):
+        model_name = "vgg16_bn"
+        extractor, _, batches = helper.create_extractor_and_dataloader(
+            model_name=model_name, pretrained=False, source="torchvision"
+        )
+        module_names = ["features.23", "classifier.3"]
+        features = extractor.extract_features(
+            batches=batches,
+            module_names=module_names,
+            flatten_acts=False,
+        )
+        return features
+
     def test_postprocessing(self):
         """Test different postprocessing methods (e.g., centering, normalization, compression)."""
         features = self.get_2D_features()
@@ -90,6 +103,26 @@ class FeaturesTestCase(unittest.TestCase):
 
             self.check_file_exists("features", format, False)
 
+    def test_storing_multi(self):
+        features = self.get_multi_features()
+        for _, feature in features.items():
+            for format in set(helper.FILE_FORMATS) - set(["txt"]):
+                # tests whether features can be saved in any of the formats except txt
+                save_features(
+                    features=feature,
+                    out_path=helper.OUT_PATH,
+                    file_format=format,
+                )
+                self.check_file_exists(f"features", format, False)
+
+    def test_extract_multi(self):
+        features = self.get_multi_features()
+        row_counts = [feature.shape[0] for feature in features.values()]
+        self.assertTrue(
+            all(count == row_counts[0] for count in row_counts),
+            "Not all features have the same number of rows!",
+        )
+
     def test_splitting_2d(self):
         n_splits = 3
         features = self.get_2D_features()
@@ -129,3 +162,29 @@ class FeaturesTestCase(unittest.TestCase):
                 file_format="txt",
                 n_splits=n_splits,
             )
+
+    def test_splitting_multi(self):
+        n_splits = 3
+        features = self.get_multi_features()
+        for format in set(helper.FILE_FORMATS) - set(["txt"]):
+            for _, feature in features.items():
+                if format == "pt":
+                    feature = torch.from_numpy(feature)
+                split_features(
+                    features=feature,
+                    root=helper.OUT_PATH,
+                    file_format=format,
+                    n_splits=n_splits,
+                )
+
+                for i in range(1, n_splits):
+                    self.check_file_exists(f"features_{i:02d}", format, False)
+
+        with self.assertRaises(Exception):
+            for _, feature in features.items():
+                split_features(
+                    features=feature,
+                    root=helper.OUT_PATH,
+                    file_format="txt",
+                    n_splits=n_splits,
+                )
