@@ -1,18 +1,25 @@
 import os
-from typing import Any, Dict
+import re
+import warnings
+from typing import Any, Callable, Dict, List, Optional, Union
 
 import numpy as np
-import timm
-import torchvision
-
 import tensorflow as tf
 import tensorflow.keras.applications as tensorflow_models
+import timm
 import torch
+import torchvision
+from tensorflow.keras.layers import Lambda
+from torch.hub import load_state_dict_from_url
 
-try:
-    from torch.hub import load_state_dict_from_url
-except ImportError:
-    from torch.utils.model_zoo import load_url as load_state_dict_from_url
+from thingsvision.utils.checkpointing import get_torch_home
+from thingsvision.utils.models.dino import vit_base, vit_small, vit_tiny
+from thingsvision.utils.models.mae import (
+    interpolate_pos_embed,
+    vit_base_patch16,
+    vit_huge_patch14,
+    vit_large_patch16,
+)
 
 from .tensorflow import TensorFlowExtractor
 from .torch import PyTorchExtractor
@@ -41,11 +48,11 @@ class TorchvisionExtractor(PyTorchExtractor):
         pretrained: bool,
         device: str,
         model_path: str = None,
-        model_parameters: Dict = None,
-        preprocess: Any = None,
+        model_parameters: Dict[str, Union[str, bool, List[str]]] = None,
+        preprocess: Optional[Callable] = None,
     ) -> None:
         model_parameters = (
-            model_parameters if model_parameters else {"weights": "DEFAULT"},
+            model_parameters if model_parameters else {"weights": "DEFAULT"}
         )
         super().__init__(
             model_name=model_name,
@@ -68,7 +75,7 @@ class TorchvisionExtractor(PyTorchExtractor):
             )
         weights = getattr(
             getattr(torchvision.models, f"{weights_name}"),
-            self.model_parameters[0]["weights"],
+            self.model_parameters["weights"],
         )
         return weights
 
@@ -95,6 +102,11 @@ class TorchvisionExtractor(PyTorchExtractor):
         apply_center_crop: bool = True,
     ) -> Any:
         if self.weights:
+            warnings.warn(
+                message="\nInput arguments are ignored because transforms are automatically inferred from model weights.\n",
+                category=UserWarning,
+                stacklevel=2,
+            )
             transforms = self.weights.transforms()
         else:
             transforms = super().get_default_transformation(
@@ -111,8 +123,8 @@ class TimmExtractor(PyTorchExtractor):
         pretrained: bool,
         device: str,
         model_path: str = None,
-        model_parameters: Dict = None,
-        preprocess: Any = None,
+        model_parameters: Dict[str, Union[str, bool, List[str]]] = None,
+        preprocess: Optional[Callable] = None,
     ) -> None:
         super().__init__(
             model_name=model_name,
@@ -125,13 +137,31 @@ class TimmExtractor(PyTorchExtractor):
 
     def load_model_from_source(self) -> None:
         """Load a (pretrained) neural network model from <timm>."""
-        if self.model_name in timm.list_models():
-            self.model = timm.create_model(self.model_name, self.pretrained)
+        if self.model_name.split(".")[0] in timm.list_models():
+            self.model = timm.create_model(self.model_name, pretrained=self.pretrained)
         else:
             raise ValueError(
                 f"\nCould not find {self.model_name} in timm library.\nChoose a different model.\n"
             )
 
+    def get_default_transformation(
+        self,
+        mean,
+        std,
+        resize_dim: int = 256,
+        crop_dim: int = 224,
+        apply_center_crop: bool = True,
+    ) -> Any:
+        warnings.warn(
+            message="\nInput arguments are ignored because <timm> automatically infers transforms from model config.\n",
+            category=UserWarning,
+            stacklevel=2,
+        )
+        data_config = timm.data.resolve_model_data_config(self.model)
+        transforms = timm.data.create_transform(**data_config, is_training=False)
+
+        return transforms
+
 
 class KerasExtractor(TensorFlowExtractor):
     def __init__(
@@ -140,8 +170,8 @@ class KerasExtractor(TensorFlowExtractor):
         pretrained: bool,
         device: str,
         model_path: str = None,
-        model_parameters: Dict = None,
-        preprocess: Any = None,
+        model_parameters: Dict[str, Union[str, bool, List[str]]] = None,
+        preprocess: Optional[Callable] = None,
     ) -> None:
         model_parameters = (
             model_parameters if model_parameters else {"weights": "imagenet"}
@@ -166,16 +196,65 @@ class KerasExtractor(TensorFlowExtractor):
             else:
                 weights = None
             self.model = model(weights=weights)
+            preproc_fun_name = self.get_keras_preprocessing(self.model_name)
+            if isinstance(preproc_fun_name, str):
+                # get preprocessing function for a specific model
+                preproc_fun = self.get_preproc_fun(preproc_fun_name)
+                # different models take differently sized inputs. this has to be accounted for.
+                resize_dim = self.model.layers[0].input_shape[0][
+                    -2
+                ]  # -2 and -3 are the H and W channel dims.
+                self.preprocess = tf.keras.Sequential(
+                    [
+                        Lambda(preproc_fun),
+                        tf.keras.layers.experimental.preprocessing.Resizing(
+                            resize_dim, resize_dim
+                        ),
+                    ]
+                )
         else:
             raise ValueError(
                 f"\nCould not find {self.model_name} among TensorFlow models.\n"
             )
 
+    @staticmethod
+    def get_preproc_fun(preproc_fun_name: str) -> Callable:
+        """Get the preprocessing function associated with a specific model."""
+        return getattr(getattr(tensorflow_models, preproc_fun_name), "preprocess_input")
+
+    def get_keras_preprocessing(self, model_name: str) -> Union[str, None]:
+        """Get the preprocessing function for the corresponding model from `tensorflow.keras.applications.*`"""
+
+        patterns = [
+            (r"^ConvNeXt(Base|Large|Small|Tiny|XLarge)$", "convnext"),
+            (r"^DenseNet\d+$", "densenet"),
+            (r"^EfficientNetB[0-7]$", "efficientnet"),
+            (r"^EfficientNetV2(B[0-3]|[LMS])$", "efficientnet_v2"),
+            (r"^InceptionResNetV2$", "inception_resnet_v2"),
+            (r"^InceptionV3$", "inception_v3"),
+            (r"^MobileNet$", "mobilenet"),
+            (r"^MobileNetV2$", "mobilenet_v2"),
+            (r"^MobileNetV3(Large|Small)$", "mobilenet_v3"),
+            (r"^NasNet(Large|Mobile)$", "nasnet"),
+            (r"^ResNet\d+$", "resnet"),
+            (r"^ResNet\d+V2$", "resnet_v2"),
+            (r"^VGG16$", "vgg16"),
+            (r"^VGG19$", "vgg19"),
+            (r"^Xception$", "xception"),
+        ]
+        # Try each pattern
+        for pattern, preproc_val in patterns:
+            if re.search(pattern, model_name):
+                return preproc_val
+
+        # If no match is found, print a warning message
+        warnings.warn(
+            f"No preprocessing function found for model {model_name}, so falling back to default preprocessing.\nOften, models that come from Keras Applications have their own preprocessing functions.\nThus, this may create inaccurate results. If you need to manually specify a preprocessing function, please do so under the `transforms` argument when creating your Dataset"
+        )
+        return None
+
 
 class SSLExtractor(PyTorchExtractor):
-    ENV_TORCH_HOME = "TORCH_HOME"
-    ENV_XDG_CACHE_HOME = "XDG_CACHE_HOME"
-    DEFAULT_CACHE_DIR = "~/.cache"
     MODELS = {
         "simclr-rn50": {
             "url": "https://dl.fbaipublicfiles.com/vissl/model_zoo/simclr_rn50_800ep_simclr_8node_resnet_16_07_20.7e8feed1/model_final_checkpoint_phase799.torch",
@@ -208,34 +287,38 @@ class SSLExtractor(PyTorchExtractor):
             "type": "vissl",
         },
         "barlowtwins-rn50": {
-            "repository": "facebookresearch/barlowtwins:main",
             "arch": "resnet50",
-            "type": "hub",
+            "type": "checkpoint_url",
+            "checkpoint_url": "https://dl.fbaipublicfiles.com/barlowtwins/ljng/resnet50.pth",
         },
         "vicreg-rn50": {
-            "repository": "facebookresearch/vicreg:main",
             "arch": "resnet50",
-            "type": "hub",
+            "type": "checkpoint_url",
+            "checkpoint_url": "https://dl.fbaipublicfiles.com/vicreg/resnet50.pth",
         },
         "dino-vit-small-p16": {
             "repository": "facebookresearch/dino:main",
             "arch": "dino_vits16",
             "type": "hub",
+            "checkpoint_url": "https://dl.fbaipublicfiles.com/dino/dino_deitsmall16_pretrain/dino_deitsmall16_pretrain.pth",
         },
-         "dino-vit-small-p8": {
+        "dino-vit-small-p8": {
             "repository": "facebookresearch/dino:main",
             "arch": "dino_vits8",
             "type": "hub",
+            "checkpoint_url": "https://dl.fbaipublicfiles.com/dino/dino_deitsmall8_pretrain/dino_deitsmall8_pretrain.pth",
         },
-        "dino-vit-big-p16": {
+        "dino-vit-base-p16": {
             "repository": "facebookresearch/dino:main",
             "arch": "dino_vitb16",
             "type": "hub",
+            "checkpoint_url": "https://dl.fbaipublicfiles.com/dino/dino_vitbase16_pretrain/dino_vitbase16_pretrain.pth",
         },
-        "dino-vit-big-p8": {
+        "dino-vit-base-p8": {
             "repository": "facebookresearch/dino:main",
             "arch": "dino_vitb8",
             "type": "hub",
+            "checkpoint_url": "https://dl.fbaipublicfiles.com/dino/dino_vitbase8_pretrain/dino_vitbase8_pretrain.pth",
         },
         "dino-xcit-small-12-p16": {
             "repository": "facebookresearch/dino:main",
@@ -262,6 +345,44 @@ class SSLExtractor(PyTorchExtractor):
             "arch": "dino_resnet50",
             "type": "hub",
         },
+        "dinov2-vit-small-p14": {
+            "repository": "facebookresearch/dinov2",
+            "arch": "dinov2_vits14",
+            "type": "hub",
+        },
+        "dinov2-vit-base-p14": {
+            "repository": "facebookresearch/dinov2",
+            "arch": "dinov2_vitb14",
+            "type": "hub",
+        },
+        "dinov2-vit-large-p14": {
+            "repository": "facebookresearch/dinov2",
+            "arch": "dinov2_vitl14",
+            "type": "hub",
+        },
+        "dinov2-vit-giant-p14": {
+            "repository": "facebookresearch/dinov2",
+            "arch": "dinov2_vitg14",
+            "type": "hub",
+        },
+        "mae-vit-base-p16": {
+            "repository": "facebookresearch/mae",
+            "arch": "mae_vit_base_patch16",
+            "type": "hub",
+            "checkpoint_url": "https://dl.fbaipublicfiles.com/mae/pretrain/mae_pretrain_vit_base.pth",
+        },
+        "mae-vit-large-p16": {
+            "repository": "facebookresearch/mae",
+            "arch": "mae_vit_large_patch16",
+            "type": "hub",
+            "checkpoint_url": "https://dl.fbaipublicfiles.com/mae/pretrain/mae_pretrain_vit_large.pth",
+        },
+        "mae-vit-huge-p14": {
+            "repository": "facebookresearch/mae",
+            "arch": "mae_vit_huge_patch14",
+            "type": "hub",
+            "checkpoint_url": "https://dl.fbaipublicfiles.com/mae/pretrain/mae_pretrain_vit_huge.pth",
+        },
     }
 
     def __init__(
@@ -270,8 +391,8 @@ class SSLExtractor(PyTorchExtractor):
         pretrained: bool,
         device: str,
         model_path: str = None,
-        model_parameters: Dict = None,
-        preprocess: Any = None,
+        model_parameters: Dict[str, Union[str, bool, List[str]]] = None,
+        preprocess: Optional[Callable] = None,
     ) -> None:
         super().__init__(
             model_name=model_name,
@@ -282,12 +403,15 @@ class SSLExtractor(PyTorchExtractor):
             device=device,
         )
 
-    def _download_and_save_model(self, model_url: str, output_model_filepath: str):
+    def _load_vissl_state_dict(self, model_url: str, unique_model_filename: str):
         """
         Downloads the model in vissl format, converts it to torchvision format and
-        saves it under output_model_filepath.
+        caches it under the unique_model_filename. Therefore, this file_name should be unique
+        per url. Otherwise, the wrong cached variant is loaded.
         """
-        model = load_state_dict_from_url(model_url, map_location=torch.device("cpu"))
+        model = load_state_dict_from_url(
+            model_url, map_location=torch.device("cpu"), file_name=unique_model_filename
+        )
 
         # get the model trunk to rename
         if "classy_state_dict" in model.keys():
@@ -298,7 +422,6 @@ class SSLExtractor(PyTorchExtractor):
             model_trunk = model
 
         converted_model = self._replace_module_prefix(model_trunk, "_feature_blocks.")
-        torch.save(converted_model, output_model_filepath)
         return converted_model
 
     def _replace_module_prefix(
@@ -317,56 +440,104 @@ class SSLExtractor(PyTorchExtractor):
         }
         return state_dict
 
-    def _get_torch_home(self):
-        """
-        Gets the torch home folder used as a cache directory for the vissl models.
-        """
-        torch_home = os.path.expanduser(
-            os.getenv(
-                SSLExtractor.ENV_TORCH_HOME,
-                os.path.join(
-                    os.getenv(
-                        SSLExtractor.ENV_XDG_CACHE_HOME, SSLExtractor.DEFAULT_CACHE_DIR
-                    ),
-                    "torch",
-                ),
-            )
-        )
-        return torch_home
-
     def load_model_from_source(self) -> None:
         """
         Load a (pretrained) neural network model from vissl. Downloads the model when it is not available.
         Otherwise, loads it from the cache directory.
         """
         if self.model_name in SSLExtractor.MODELS:
+
+            # unique model id name for all models
+            unique_model_filename = f"thingsvision_ssl_v0_{self.model_name}.pth"
+
+            # defines how the model should be loaded
             model_config = SSLExtractor.MODELS[self.model_name]
+
+            # VISSL MODELS
             if model_config["type"] == "vissl":
-                cache_dir = os.path.join(self._get_torch_home(), "vissl")
-                model_filepath = os.path.join(cache_dir, self.model_name + ".torch")
-                if not os.path.exists(model_filepath):
-                    os.makedirs(cache_dir, exist_ok=True)
-                    model_state_dict = self._download_and_save_model(
-                        model_url=model_config["url"],
-                        output_model_filepath=model_filepath,
-                    )
-                else:
-                    model_state_dict = torch.load(
-                        model_filepath, map_location=torch.device("cpu")
-                    )
+                model_state_dict = self._load_vissl_state_dict(
+                    model_url=model_config["url"],
+                    unique_model_filename=unique_model_filename,
+                )
                 self.model = getattr(torchvision.models, model_config["arch"])()
-                if model_config["arch"] == 'resnet50':
+                if model_config["arch"] == "resnet50":
                     self.model.fc = torch.nn.Identity()
                 self.model.load_state_dict(model_state_dict, strict=True)
+
+            # HUB MODELS
             elif model_config["type"] == "hub":
-                self.model = torch.hub.load(
-                    model_config["repository"], model_config["arch"]
-                )
-                if model_config["arch"] == 'resnet50':
+                if self.model_name.startswith("dino-vit"):
+                    if self.model_name == "dino-vit-tiny-p8":
+                        model = vit_tiny(patch_size=8)
+                    elif self.model_name == "dino-vit-tiny-p16":
+                        model = vit_tiny(patch_size=16)
+                    elif self.model_name == "dino-vit-small-p8":
+                        model = vit_small(patch_size=8)
+                    elif self.model_name == "dino-vit-small-p16":
+                        model = vit_small(patch_size=16)
+                    elif self.model_name == "dino-vit-base-p8":
+                        model = vit_base(patch_size=8)
+                    elif self.model_name == "dino-vit-base-p16":
+                        model = vit_base(patch_size=16)
+                    else:
+                        raise ValueError(f"\n{self.model_name} is not available.\n")
+                    state_dict = torch.hub.load_state_dict_from_url(
+                        model_config["checkpoint_url"],
+                        map_location=torch.device("cpu"),
+                        # This is used to cache the file
+                        file_name=unique_model_filename,
+                    )
+                    model.load_state_dict(state_dict, strict=True)
+                    self.model = model
+                elif self.model_name.startswith("mae"):
+                    if self.model_name == "mae-vit-base-p16":
+                        model = vit_base_patch16(num_classes=0, drop_rate=0.0)
+                    elif self.model_name == "mae-vit-large-p16":
+                        model = vit_large_patch16(num_classes=0, drop_rate=0.0)
+                    elif self.model_name == "mae-vit-huge-p14":
+                        model = vit_huge_patch14(num_classes=0, drop_rate=0.0)
+                    else:
+                        raise ValueError(f"\n{self.model_name} is not available.\n")
+                    state_dict = torch.hub.load_state_dict_from_url(
+                        model_config["checkpoint_url"],
+                        map_location=torch.device("cpu"),
+                        file_name=unique_model_filename,
+                    )
+                    checkpoint_model = state_dict["model"]
+                    # interpolate position embedding
+                    interpolate_pos_embed(model, checkpoint_model)
+                    model.load_state_dict(checkpoint_model, strict=False)
+                    self.model = model
+                else:
+                    self.model = torch.hub.load(
+                        model_config["repository"], model_config["arch"]
+                    )
+                    if model_config["arch"] == "resnet50":
+                        self.model.fc = torch.nn.Identity()
+
+            # MODELS FROM CHECKPOINT URL
+            elif model_config["type"] == "checkpoint_url":
+
+                # load architecture
+                self.model = getattr(torchvision.models, model_config["arch"])()
+                if model_config["arch"] == "resnet50":
                     self.model.fc = torch.nn.Identity()
+
+                # load and cache state_dict
+                state_dict = torch.hub.load_state_dict_from_url(
+                    model_config["checkpoint_url"],
+                    map_location=torch.device("cpu"),
+                    # IMPORTANT that this is unique as it will be used for caching
+                    file_name=unique_model_filename,
+                )
+
+                # load state dict to model
+                self.model.load_state_dict(state_dict, strict=True)
+
             else:
-                raise ValueError(f"\nUnknown model type.\n")
+                type = model_config["type"]
+                raise ValueError(f"\nUnknown model type: {type}.\n")
         else:
             raise ValueError(
-                f"\nCould not find {self.model_name} in the SSLExtractor.\n"
+                f"\nCould not find {self.model_name} in the SSLExtractor.\nUse a different model.\n"
             )
