@@ -1,8 +1,9 @@
 from .alexnet_ecoset import Alexnet_ecoset
 from .alexnet_salobjsub import AlexNet_SalObjSub
-from .resnet50_ecoset import Resnet50_ecoset
-from .vgg16_ecoset import VGG16_ecoset
-from .openclip import OpenCLIP
 from .inception_ecoset import Inception_ecoset
 from .official_clip import clip
-from .harmonization import Harmonization
+from .openclip import OpenCLIP
+from .resnet50_ecoset import Resnet50_ecoset
+from .vgg16_ecoset import VGG16_ecoset
+from .sam import SegmentAnything
+from .align import Kakaobrain_Align
