@@ -1,7 +1,8 @@
+from typing import Any
+
 import torch
 import torchvision.models as models
 
-from typing import Any
 from .custom import Custom
 
 
@@ -12,7 +13,7 @@ class VGG16bn_ecoset(Custom):
 
     def create_model(self) -> Any:
         model = models.vgg16_bn(weights=None, num_classes=565)
-        path_to_weights = "https://osf.io/z5uf3/download"
+        path_to_weights = "https://osf.io/fe7s5/download"
         state_dict = torch.hub.load_state_dict_from_url(
             path_to_weights, map_location=self.device, file_name="VGG16bn_ecoset"
         )
